@@ -6,10 +6,10 @@ package cmdapi
 
 import (
 	"context"
+	"database/sql"
 	"errors"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"net/url"
 	"os"
 	"os/exec"
@@ -36,6 +36,39 @@ func TestMigrate(t *testing.T) {
 	require.NoError(t, err)
 }
 
+func TestMigrate_Import(t *testing.T) {
+	for _, tool := range []string{"dbmate", "flyway", "golang-migrate", "goose", "liquibase"} {
+		p := t.TempDir()
+		t.Run(tool, func(t *testing.T) {
+			path := filepath.FromSlash("testdata/import/" + tool)
+			out, err := runCmd(
+				Root, "migrate", "import",
+				"--from", "file://"+path,
+				"--to", "file://"+p,
+				"--dir-format", tool,
+			)
+			require.NoError(t, err)
+			require.Zero(t, out)
+
+			path += "_gold"
+			ex, err := os.ReadDir(path)
+			require.NoError(t, err)
+			ac, err := os.ReadDir(p)
+			require.NoError(t, err)
+			require.Equal(t, len(ex)+1, len(ac)) // sum file
+
+			for i := range ex {
+				e, err := os.ReadFile(filepath.Join(path, ex[i].Name()))
+				require.NoError(t, err)
+				a, err := os.ReadFile(filepath.Join(p, ex[i].Name()))
+				require.NoError(t, err)
+
+				require.Equal(t, string(e), string(a))
+			}
+		})
+	}
+}
+
 func TestMigrate_Apply(t *testing.T) {
 	var (
 		p   = t.TempDir()
@@ -52,7 +85,7 @@ func TestMigrate_Apply(t *testing.T) {
 		"-u", openSQLite(t, ""),
 	)
 	require.NoError(t, err)
-	require.Equal(t, "The migration directory is synced with the database, no migration files to execute\n", s)
+	require.Equal(t, "No migration files to execute\n", s)
 
 	// Fails on directory without sum file.
 	require.NoError(t, os.Rename(
@@ -190,12 +223,12 @@ func TestMigrate_Apply(t *testing.T) {
 		"--url", fmt.Sprintf("sqlite://file:%s?cache=shared&_fk=1", filepath.Join(p, "test2.db")),
 		"--tx-mode", "none",
 	)
-	require.NoError(t, exec.Command("cp", "-r", "testdata/sqlite2", "testdata/sqlite3").Run())
 	t.Cleanup(func() {
-		require.NoError(t, os.RemoveAll("testdata/sqlite3"))
+		_ = os.RemoveAll("testdata/sqlite3")
 	})
+	require.NoError(t, exec.Command("cp", "-r", "testdata/sqlite2", "testdata/sqlite3").Run())
 	sed(t, "s/col_2/col_5/g", "testdata/sqlite3/20220318104615_second.sql")
-	_, err = runCmd(Root, "migrate", "hash", "--force", "--dir", "file://testdata/sqlite3")
+	_, err = runCmd(Root, "migrate", "hash", "--dir", "file://testdata/sqlite3")
 	require.NoError(t, err)
 	s, err = runCmd(
 		Root, "migrate", "apply",
@@ -207,7 +240,7 @@ func TestMigrate_Apply(t *testing.T) {
 	// Fixing the migration file will finish without errors.
 	sed(t, "s/col_5/col_2/g", "testdata/sqlite3/20220318104615_second.sql")
 	sed(t, "s/asdasd //g", "testdata/sqlite3/20220318104615_second.sql")
-	_, err = runCmd(Root, "migrate", "hash", "--force", "--dir", "file://testdata/sqlite3")
+	_, err = runCmd(Root, "migrate", "hash", "--dir", "file://testdata/sqlite3")
 	require.NoError(t, err)
 	s, err = runCmd(
 		Root, "migrate", "apply",
@@ -230,10 +263,10 @@ func TestMigrate_Apply(t *testing.T) {
 		"--url", fmt.Sprintf("sqlite://file:%s?cache=shared&_fk=1", filepath.Join(p, "test2.db")),
 	)
 	require.NoError(t, err)
-	require.Equal(t, "The migration directory is synced with the database, no migration files to execute\n", s)
+	require.Equal(t, "No migration files to execute\n", s)
 
-	// Dry run will print the statements in second migration file without executing them. No changes to the revisions
-	// will be done.
+	// Dry run will print the statements in second migration file without executing them.
+	// No changes to the revisions will be done.
 	s, err = runCmd(
 		Root, "migrate", "apply",
 		"--dir", "file://testdata/sqlite",
@@ -259,6 +292,113 @@ func TestMigrate_Apply(t *testing.T) {
 	revs, err = rrw.ReadRevisions(ctx)
 	require.NoError(t, err)
 	require.Len(t, revs, 1)
+	MigrateFlags.Apply.DryRun = false // global flag, undo for rest of tests
+
+	// Prerequisites for testing missing migration behavior.
+	c1, err = sqlclient.Open(ctx, fmt.Sprintf("sqlite://file:%s?cache=shared&_fk=1", filepath.Join(p, "test3.db")))
+	require.NoError(t, err)
+	t.Cleanup(func() {
+		require.NoError(t, c1.Close())
+	})
+	require.NoError(t, os.Rename(
+		"testdata/sqlite3/20220318104615_second.sql",
+		"testdata/sqlite3/20220318104616_second.sql",
+	))
+	_, err = runCmd(Root, "migrate", "hash", "--dir", "file://testdata/sqlite3")
+	require.NoError(t, err)
+	rrw, err = migrate2.NewEntRevisions(ctx, c1)
+	require.NoError(t, err)
+	require.NoError(t, rrw.Migrate(ctx))
+
+	// No changes if the last revision has a greater version than the last migration.
+	require.NoError(t, rrw.WriteRevision(ctx, &migrate.Revision{Version: "zzz"}))
+	s, err = runCmd(
+		Root, "migrate", "apply",
+		"--dir", "file://testdata/sqlite3",
+		"--url", fmt.Sprintf("sqlite://file:%s?cache=shared&_fk=1", filepath.Join(p, "test3.db")),
+	)
+	require.NoError(t, err)
+	require.Equal(t, "No migration files to execute\n", s)
+
+	// If the revision is before the last but after the first migration, only the last one is pending.
+	_, err = c1.ExecContext(ctx, "DROP table `atlas_schema_revisions`")
+	require.NoError(t, err)
+	s, err = runCmd(
+		Root, "migrate", "apply", "1",
+		"--dir", "file://testdata/sqlite3",
+		"--url", fmt.Sprintf("sqlite://file:%s?cache=shared&_fk=1", filepath.Join(p, "test3.db")),
+	)
+	require.NoError(t, rrw.WriteRevision(ctx, &migrate.Revision{Version: "20220318104615"}))
+	require.NoError(t, err)
+	s, err = runCmd(
+		Root, "migrate", "apply",
+		"--dir", "file://testdata/sqlite3",
+		"--url", fmt.Sprintf("sqlite://file:%s?cache=shared&_fk=1", filepath.Join(p, "test3.db")),
+	)
+	require.NoError(t, err)
+	require.NotContains(t, s, "20220318104614")                     // log to version
+	require.Contains(t, s, "20220318104616")                        // log to version
+	require.Contains(t, s, "ALTER TABLE `tbl` ADD `col_2` bigint;") // logs statement
+
+	// If the revision is before every migration file, every file is pending.
+	_, err = c1.ExecContext(ctx, "DROP table `atlas_schema_revisions`; DROP table `tbl`;")
+	require.NoError(t, err)
+	require.NoError(t, rrw.Migrate(ctx))
+	require.NoError(t, rrw.WriteRevision(ctx, &migrate.Revision{Version: "1"}))
+	require.NoError(t, err)
+	s, err = runCmd(
+		Root, "migrate", "apply",
+		"--dir", "file://testdata/sqlite3",
+		"--url", fmt.Sprintf("sqlite://file:%s?cache=shared&_fk=1", filepath.Join(p, "test3.db")),
+	)
+	require.NoError(t, err)
+	require.Contains(t, s, "20220318104614")                         // log to version
+	require.Contains(t, s, "20220318104616")                         // log to version
+	require.Contains(t, s, "CREATE TABLE tbl (`col` int NOT NULL);") // logs statement
+	require.Contains(t, s, "ALTER TABLE `tbl` ADD `col_2` bigint;")  // logs statement
+
+	// If the revision is partially applied, error out.
+	require.NoError(t, rrw.WriteRevision(ctx, &migrate.Revision{Version: "z", Description: "z", Total: 1}))
+	require.NoError(t, err)
+	_, err = runCmd(
+		Root, "migrate", "apply",
+		"--dir", "file://testdata/sqlite3",
+		"--url", fmt.Sprintf("sqlite://file:%s?cache=shared&_fk=1", filepath.Join(p, "test3.db")),
+	)
+	require.EqualError(t, err, migrate.MissingMigrationError{Version: "z", Description: "z"}.Error())
+}
+
+func TestMigrate_ApplyTxMode(t *testing.T) {
+	for _, mode := range []string{"none", "file", "all"} {
+		t.Run(mode, func(t *testing.T) {
+			p := t.TempDir()
+			// Apply the first 2 migrations.
+			s, err := runCmd(
+				Root, "migrate", "apply",
+				"--dir", "file://testdata/sqlitetx",
+				"--url", fmt.Sprintf("sqlite://file:%s?cache=shared&_fk=1", filepath.Join(p, "test.db")),
+				"--tx-mode", mode,
+				"2",
+			)
+			require.NoError(t, err)
+			require.NotEmpty(t, s)
+			db, err := sql.Open("sqlite3", fmt.Sprintf("file:%s?cache=shared&_fk=1", filepath.Join(p, "test.db")))
+			require.NoError(t, err)
+			var n int
+			require.NoError(t, db.QueryRow("SELECT COUNT(*) FROM `friendships`").Scan(&n))
+			require.Equal(t, 2, n)
+
+			// Apply the rest.
+			s, err = runCmd(
+				Root, "migrate", "apply",
+				"--dir", "file://testdata/sqlitetx",
+				"--url", fmt.Sprintf("sqlite://file:%s?cache=shared&_fk=1", filepath.Join(p, "test.db")),
+				"--tx-mode", mode,
+			)
+			require.NoError(t, db.QueryRow("SELECT COUNT(*) FROM `friendships`").Scan(&n))
+			require.Equal(t, 2, n)
+		})
+	}
 }
 
 func TestMigrate_ApplyBaseline(t *testing.T) {
@@ -271,7 +411,7 @@ func TestMigrate_ApplyBaseline(t *testing.T) {
 		"--url", fmt.Sprintf("sqlite://file:%s?cache=shared&_fk=1", filepath.Join(p, "test.db")),
 	)
 	require.NoError(t, err)
-	require.Contains(t, s, "The migration directory is synced with the database, no migration files to execute")
+	require.Contains(t, s, "No migration files to execute")
 
 	// Next run without baseline should run the migration from the baseline.
 	s, err = runCmd(
@@ -288,7 +428,6 @@ func TestMigrate_Diff(t *testing.T) {
 	to := hclURL(t)
 
 	// Will create migration directory if not existing.
-	MigrateFlags.Force = false
 	_, err := runCmd(
 		Root, "migrate", "diff",
 		"name",
@@ -432,7 +571,7 @@ func TestMigrate_Validate(t *testing.T) {
 	p := t.TempDir()
 	require.NoError(t, os.WriteFile(filepath.Join(p, "1_initial.sql"), []byte("create table t1 (c1 int)"), 0644))
 	require.NoError(t, os.WriteFile(filepath.Join(p, "2_second.sql"), []byte("create table t2 (c2 int)"), 0644))
-	_, err = runCmd(Root, "migrate", "hash", "--force", "--dir", "file://"+p)
+	_, err = runCmd(Root, "migrate", "hash", "--dir", "file://"+p)
 	require.NoError(t, err)
 	s, err = runCmd(
 		Root, "migrate", "validate",
@@ -452,15 +591,20 @@ func TestMigrate_Hash(t *testing.T) {
 	require.Zero(t, s)
 	require.NoError(t, err)
 
+	// Prints a warning if --force flag is still used.
+	s, err = runCmd(Root, "migrate", "hash", "--dir", "file://testdata/mysql", "--force")
+	require.NoError(t, err)
+	require.Equal(t, "Flag --force has been deprecated, you can safely omit it.\n", s)
+
 	p := t.TempDir()
 	err = copyFile(filepath.Join("testdata", "mysql", "20220318104614_initial.sql"), filepath.Join(p, "20220318104614_initial.sql"))
 	require.NoError(t, err)
 
-	s, err = runCmd(Root, "migrate", "hash", "--dir", "file://"+p, "--force")
+	s, err = runCmd(Root, "migrate", "hash", "--dir", "file://"+p)
 	require.Zero(t, s)
 	require.NoError(t, err)
 	require.FileExists(t, filepath.Join(p, "atlas.sum"))
-	d, err := ioutil.ReadFile(filepath.Join(p, "atlas.sum"))
+	d, err := os.ReadFile(filepath.Join(p, "atlas.sum"))
 	require.NoError(t, err)
 	dir, err := migrate.NewLocalDir(p)
 	require.NoError(t, err)
